mean.CRPS <- function(y0, x0, m = c(1,2,3,4,5), gamma_est, sigma_est, beta_est = rep(NA,length(gamma_est)), realLOD = NA){
  # x0: covariate
  # y0: observed response
  # Returns corresponding CRPS for all pairs (y0,x0)
  
  p_est = pnorm(x0 %*% beta_est)
  mu_est = x0 %*% gamma_est
  
  if(m == 1){ # Mixture w/ IC
    s = mean(mapply(intfunc, y0 = y0, p_est = p_est, mu_est = mu_est, MoreArgs = list(C=C1, realLOD=realLOD, sigma_est=sigma_est)))
  }else if(m == 2){ # Mixture w/o IC
    s = mean(mapply(intfunc, y0 = y0, p_est = p_est, mu_est = mu_est, MoreArgs = list(C=C2, realLOD=realLOD, sigma_est=sigma_est)))
  }else if(m == 3){ # Substitute
    s = mean(mapply(intfunc, y0 = y0, p_est = p_est, mu_est = mu_est, MoreArgs = list(C=C3, realLOD=realLOD, sigma_est=sigma_est)))
  }else if(m == 4){ # Tobit
    s = mean(mapply(intfunc, y0 = y0, p_est = p_est, mu_est = mu_est, MoreArgs = list(C=C4, realLOD=realLOD, sigma_est=sigma_est)))
  }
  return(s)
}


intfunc <- function(y0, p_est, mu_est, sigma_est, C, realLOD){
  return(integrate(C, lower = 0, upper = Inf, y0, p = p_est, mu = mu_est, sigma_est = sigma_est, realLOD = realLOD)$value)
}

# TPIC
C1 <- function(z, y0, p, mu, sigma_est, realLOD){
  c  = numeric(length(z))
  
  c[which(z < 0)] = (0 - as.integer(z[which(z < 0)] >= y0))^2
  c[which(z >= 0 & z < realLOD)] = (p + (1-p)*plnorm(realLOD, mu, sigma_est) - as.integer(z[which(z >= 0 & z < realLOD)] >= y0))^2
  c[which(z >= realLOD)] = (p + (1-p)*plnorm(z[which(z >= realLOD)], mu, sigma_est) - as.integer(z[which(z >= realLOD)] >= y0))^2
  return(c)
}

# TP
C2 <- function(z, y0, p, mu, sigma_est, realLOD){
  c  = numeric(length(z))
  
  c[which(z < 0)] = (0 - as.integer(z[which(z < 0)] >= y0))^2
  c[which(z >= 0)] = (p + (1-p)*plnorm(z[which(z >= 0)], mu, sigma_est) - as.integer(z[which(z >= 0)] >= y0))^2
  
  return(c)
}

#SUB
C3 <- function(z, y0, p, mu, sigma_est, realLOD){
  c  = (plnorm(z, mu, sigma_est) - as.integer(z >= y0))^2
  return(c)
}

# TOBIT
C4 <- function(z, y0, p, mu, sigma_est, realLOD){
  c  = numeric(length(z))
  
  c[which(z < 0)] = (0 - as.integer(z[which(z < 0)] >= y0))^2
  c[which(z >= 0 & z < realLOD)] = (plnorm(realLOD, mu, sigma_est) - as.integer(z[which(z >= 0 & z < realLOD)] >= y0))^2
  c[which(z >= realLOD)] = (plnorm(z[which(z >= realLOD)], mu, sigma_est) -  as.integer(z[which(z >= realLOD)] >= y0))^2
  return(c)
}

