fit_bivariate = function(y, par, T1, T2, parts){
  y1 = y[,1]
  y2 = y[,2]
  if(parts == 1){
    return(optim(par = par, fn = optim_bitobit, y1 = y1, y2 = y2, T1 = T1, T2 = T2, 
                 method = "BFGS", hessian = TRUE, control = list("fnscale"=-1))) 
  }else if(parts == 2){
    return(optim(par = par, fn = optim_bitp, y1 = y1, y2 = y2, T1 = T1, T2 = T2, 
                 method = "BFGS", hessian = TRUE, control = list("fnscale"=-1))) 
  }else if(parts == 4){
    return(optim(par = par, fn = optim_fp, y1 = y1, y2 = y2, T1 = T1, T2 = T2, 
                 method = "BFGS", hessian = TRUE, control = list("fnscale"=-1))) 
  }else if(parts == 0){ # Only fully observed pairs
    return(optim(par = par, fn = optim_obs, y1 = y1, y2 = y2, 
                 method = "BFGS", hessian = TRUE, control = list("fnscale"=-1))) 
  }
}


####### TOBIT 
log_likelihood_bitobit = function(y1, y2,mu1,mu2,sigma1,sigma2,mu_rho,T1,T2){
  sigma1 = abs(sigma1)
  sigma2 = abs(sigma2)
  rho = 2*pnorm(mu_rho)-1
  
  sigma12 = sqrt(sigma1^2*(1-rho^2))
  sigma21 = sqrt(sigma2^2*(1-rho^2))
  
  likelihood = 0
  for(i in 1:length(y1)){
    y1i = y1[i]
    y2i = y2[i]
    
    mu12 = mu1 + rho*sigma1/sigma2 * (y2i- mu2)
    mu21 = mu2 + rho*sigma2/sigma1 * (y1i- mu1)
    
    if(!is.na(y1i) & !is.na(y2i)){
      update = dnorm(y1i, mean = mu12, sd = sigma12, log = TRUE) +  dnorm(y2i, mean = mu2, sd = sigma2, log = TRUE)
    }else if(!is.na(y2i)){
      update = dnorm(y2i, mean = mu2, sd = sigma2, log = TRUE) + pnorm(T1, mean = mu12, sd = sigma12, log = TRUE)
    }else if(!is.na(y1i)){
      update = dnorm(y1i, mean = mu1, sd = sigma1, log = TRUE) + pnorm(T2, mean = mu21, sd = sigma21, log = TRUE)
    }else{
      Sigma = matrix(c(sigma1^2,rho*sigma1*sigma2,rho*sigma1*sigma2,sigma2^2), ncol = 2)
      update = log(pmvnorm(lower = c(-Inf,-Inf), upper = c(T1,T2), mean = c(mu1,mu2), sigma = Sigma))
    }
    likelihood = likelihood + update
  }
  return(likelihood)
}

optim_bitobit = function(par, T1, T2, y1, y2){
  mu1 = par[1]
  mu2 = par[2]
  sigma1 = par[3]
  sigma2 = par[4]
  mu_rho = par[5]
  return(log_likelihood_bitobit(y1, y2,mu1,mu2,sigma1,sigma2,mu_rho,T1,T2))
}


###### TOWPART

log_likelihood_bitp = function(y1, y2,mu1,mu2,sigma1,sigma2,mu_rho,mu_pi,T1,T2){
  sigma1 = abs(sigma1)
  sigma2 = abs(sigma2)
  pi = pnorm(mu_pi)
  rho = 2*pnorm(mu_rho)-1
  
  
  sigma12 = sqrt(sigma1^2*(1-rho^2))
  sigma21 = sqrt(sigma2^2*(1-rho^2))
  
  likelihood = 0
  for(i in 1:length(y1)){
    y1i = y1[i]
    y2i = y2[i]
    
    mu12 = mu1 + rho*sigma1/sigma2 * (y2i- mu2)
    mu21 = mu2 + rho*sigma2/sigma1 * (y1i- mu1)
    
    if(!is.na(y1i) & !is.na(y2i)){
      update = log(1-pi) + dnorm(y1i, mean = mu12, sd = sigma12, log = TRUE) +  dnorm(y2i, mean = mu2, sd = sigma2, log = TRUE)
    }else if(!is.na(y2i)){
      update = log(1-pi) + dnorm(y2i, mean = mu2, sd = sigma2, log = TRUE) + pnorm(T1, mean = mu12, sd = sigma12, log = TRUE)
    }else if(!is.na(y1i)){
      update = log(1-pi) + dnorm(y1i, mean = mu1, sd = sigma1, log = TRUE) + pnorm(T2, mean = mu21, sd = sigma21, log = TRUE)
    }else{
      Sigma = matrix(c(sigma1^2,rho*sigma1*sigma2,rho*sigma1*sigma2,sigma2^2), ncol = 2)
      update = log(pi + (1-pi)*pmvnorm(lower = c(-Inf,-Inf), upper = c(T1,T2), mean = c(mu1,mu2), sigma = Sigma))
    }
    likelihood = likelihood + update
  }
  return(likelihood)
}

optim_bitp = function(par, T1, T2, y1, y2){
  mu1 = par[1]
  mu2 = par[2]
  sigma1 = par[3]
  sigma2 = par[4]
  mu_rho = par[5]
  mu_pi = par[6]
  return(log_likelihood_bitp(y1, y2,mu1,mu2,sigma1,sigma2,mu_rho,mu_pi,T1,T2))
}



###### FOUR-PART

log_likelihood_fp = function(y1,y2,mu1,mu2,sigma1,sigma2,mu_rho,mu_pi1,mu_pi2,mu_pi3,mu1L,sigma1L,mu2L,sigma2L,T1,T2){
  sigma1 = abs(sigma1)
  sigma2 = abs(sigma2)
  sigma1L = abs(sigma1L)
  sigma2L = abs(sigma2L)
  pi1 = exp(mu_pi1)/(exp(mu_pi1) + exp(mu_pi2) + exp(mu_pi3) + 1)
  pi2 = exp(mu_pi2)/(exp(mu_pi1) + exp(mu_pi2) + exp(mu_pi3) + 1)
  pi3 = exp(mu_pi3)/(exp(mu_pi1) + exp(mu_pi2) + exp(mu_pi3) + 1)
  #print(pi1 + pi2 + pi3)
  rho = 2*pnorm(mu_rho)-1
  
  sigma12 = sqrt(sigma1^2*(1-rho^2))
  sigma21 = sqrt(sigma2^2*(1-rho^2))
  
  likelihood = 0
  for(i in 1:length(y1)){
    y1i = y1[i]
    y2i = y2[i]
    
    mu12 = mu1 + rho*sigma1/sigma2 * (y2i- mu2)
    mu21 = mu2 + rho*sigma2/sigma1 * (y1i- mu1)
    
    if(!is.na(y1i) & !is.na(y2i)){ # Both observed
      update = log(1-pi1-pi2-pi3) + dnorm(y1i, mean = mu12, sd = sigma12, log = TRUE) +  dnorm(y2i, mean = mu2, sd = sigma2, log = TRUE)
    }else if(!is.na(y1i)){ # y1 Observed
      update = log((1-pi1-pi2-pi3)*dnorm(y1i, mean = mu1, sd = sigma1)*pnorm(T2, mean = mu21, sd = sigma21) + pi1*dnorm(y1i,mean=mu1L,sd=sigma1L))
    }else if(!is.na(y2i)){ # y2 Observed
      update = log((1-pi1-pi2-pi3)*dnorm(y2i, mean = mu2, sd = sigma2)*pnorm(T1, mean = mu12, sd = sigma12) + pi2*dnorm(y2i,mean=mu2L,sd=sigma2L))
    }else{ # Both censored
      Sigma = matrix(c(sigma1^2,rho*sigma1*sigma2,rho*sigma1*sigma2,sigma2^2), ncol = 2)
      update = log(pi3 + pi1*pnorm(T1, mean = mu1L, sd = sigma1L) + pi2*pnorm(T2, mean = mu2L, sd=sigma2L) + (1-pi1-pi2-pi3)*pmvnorm(lower = c(-Inf,-Inf), upper = c(T1,T2), mean = c(mu1,mu2), sigma = Sigma))
    }
    likelihood = likelihood + update
  }
  return(likelihood)
}

optim_fp = function(par, T1, T2, y1, y2){
  mu_pi1 = par[1]
  mu_pi2 = par[2]
  mu_pi3 = par[3]
  mu1 = par[4]
  mu2 = par[5]
  sigma1 = par[6]
  sigma2 = par[7]
  mu_rho = par[8]
  mu1L = par[9]
  sigma1L = par[10]
  mu2L = par[11]
  sigma2L = par[12]
  
  mu1L = mu1
  sigma1L = sigma1
  mu2L = mu2
  sigma2L = sigma2
  return(log_likelihood_fp(y1,y2,mu1,mu2,sigma1,sigma2,mu_rho,mu_pi1,mu_pi2,mu_pi3,mu1L,sigma1L,mu2L,sigma2L,T1,T2))
}


####### Observed 
log_likelihood_obs = function(y1, y2,mu1,mu2,sigma1,sigma2,mu_rho){
  sigma1 = abs(sigma1)
  sigma2 = abs(sigma2)
  rho = 2*pnorm(mu_rho)-1
  
  sigma12 = sqrt(sigma1^2*(1-rho^2))
  
  likelihood = 0
  for(i in 1:length(y1)){
    y1i = y1[i]
    y2i = y2[i]
    
    mu12 = mu1 + rho*sigma1/sigma2 * (y2i- mu2)
    
    update = dnorm(y1i, mean = mu12, sd = sigma12, log = TRUE) +  dnorm(y2i, mean = mu2, sd = sigma2, log = TRUE)
    likelihood = likelihood + update
  }
  return(likelihood)
}

optim_obs = function(par, y1, y2){
  mu1 = par[1]
  mu2 = par[2]
  sigma1 = par[3]
  sigma2 = par[4]
  mu_rho = par[5]
  return(log_likelihood_obs(y1, y2,mu1,mu2,sigma1,sigma2,mu_rho))
}

