/* TPIC WITH SKEW */
proc nlmixed
data = multiplex
cov
df=1000
alpha=0.05
tech= QUANEW
update = BFGS;
parms
g0 = 2.41 ga = 0 gr = 0.33 gs = 0.53 gt1 = 0.91 gt2 = 0.92 
gt1r = -0.97 gt2r = -1.25 gt1s = -1.44 gt2s = -1.41
sigma = 0.51 delta = -1.61
b0 = -0.67 br = -0.95 bs = -0.53 bt1 = -1.03 bt2 = -0.34  ba = 0.02
bt1r = 1.17 bt2r = -0.37 bt1s = 0.94 bt2s = -0.05
s11 = 0.5 s12 = 0 s22 = 0.5;

bounds sigma > 0; bounds s11 > 0; bounds s22 > 0;

T = log(0.29); *LOD;
censored = (y=-100); *Indicator variable;

mu = g0 + gr*(RA+SN_RA) + gs*SLE + ga*age + 
gt1*(time3+time4) + gt2*(time5+time6) + 
gt1r*(time3+time4)*(RA+SN_RA) + gt2r*(time5+time6)*(RA+SN_RA) + 
gt1s*(time3+time4)*SLE + gt2s*(time5+time6)*SLE + t1; 

mu_pi = b0 + br*(RA+SN_RA) + bs*SLE + ba*age + 
bt1*(time3+time4) + bt2*(time5+time6) + 
bt1r*(time3+time4)*(RA+SN_RA) + bt2r*(time5+time6)*(RA+SN_RA) + 
bt1s*(time3+time4)*SLE + bt2s*(time5+time6)*SLE + t2; 

pi = CDF('normal', mu_pi);

* Estimate Owenst T-function;
h = (T - mu)/sqrt(sigma**2 + delta**2);
a = delta/sigma;
jmax = 50;
cut.point = 8;

aa = abs(a);
ah = abs(h);
if (aa = .I) then OT = sign(a) * 0.5*CDF('normal',-ah);
else if (aa = 0) then OT = 0;
else if (ah = .I) then OT = 0;
else if (aa <= 1) then do;
	th = ah;
	ta = aa;
	
	* Two ways to approximate integral based on cutpoint;
	low = (th <= cut.point);
	if low then do;
		matr = 0;
		cumb = 0;
			do j = 0 to jmax;
				cumb = cumb + (th**(2*j)) / ((2**j) * GAMMA(j+1));
				b1 = EXP(-0.5 * th**2) * cumb;
				matr = matr + (-1)**j * ta**(2*j+1) / (2*j+1) * (1-b1);
			end; *for;
		T.int = (ATAN(ta) - matr) / (2*CONSTANT("pi"));
		OT = SIGN(a) * T.int;
		end; *if;
	else OT = SIGN(a) * (ATAN(ta) * EXP(-0.5*(th**2)*ta/ATAN(ta)) * (1+0.00868*(th*ta)**4) / (2*CONSTANT("pi")));
end; *if;
else do; *(aa > 1);
	th = aa * ah;
	ta = 1 / aa;
	
	* Two ways to approximate integral based on cutpoint;
	low = (th <= cut.point);
	if low then do;
		matr = 0;
		cumb = 0;
			do j = 0 to jmax;
				cumb = cumb + (th**(2*j)) / ((2**j) * GAMMA(j+1));
				b1 = EXP(-0.5 * th**2) * cumb;
				matr = matr + (-1)**j * ta**(2*j+1) / (2*j+1) * (1-b1);
			end; *for;
		T.int = (ATAN(ta) - matr) / (2*CONSTANT("pi"));
		end; *if;
	else T.int = ATAN(ta) * EXP(-0.5*(th**2)*ta/ATAN(ta)) * (1+0.00868*(th*ta)**4) / (2*CONSTANT("pi"));

	OT = sign(a) * (0.5*CDF('normal',ah) + CDF('normal',aa*ah)*(0.5-CDF('normal',ah)) - T.int);
end;

w = sqrt(sigma**2 + delta**2);
if censored then p = pi + (1-pi)*(CDF('normal',(T-mu)/w) - 2*OT);
else p = (1-pi)*2/w * PDF('normal',(y-mu)/w) * CDF('normal',delta/sigma*(y-mu)/w);
loglike = log(p);

model y ~ general (loglike);
random t1 t2 ~ normal([0,0],[s11, s12, s22]) subject = individual;
run;


/* TP WITH SKEW */
proc nlmixed
data = multiplex
cov
df=1000
alpha=0.05
gconv = 0
tech= QUANEW
update = BFGS;
parms
g0 = 2.41 ga = 0 gr = 0.33 gs = 0.53 gt1 = 0.91 gt2 = 0.92 
gt1r = -0.97 gt2r = -1.25 gt1s = -1.44 gt2s = -1.41
sigma = 0.51 delta = -1.61
b0 = -0.67 br = -0.95 bs = -0.53 bt1 = -1.03 bt2 = -0.34  ba = 0.02
bt1r = 1.17 bt2r = -0.37 bt1s = 0.94 bt2s = -0.05
s11 = 0.5 s12 = 0 s22 = 0.5;

bounds sigma > 0; bounds s11 > 0; bounds s22 > 0;

censored = (y=-100); *Indicator variable;

mu = g0 + gr*(RA+SN_RA) + gs*SLE + ga*age + 
gt1*(time3+time4) + gt2*(time5+time6) + 
gt1r*(time3+time4)*(RA+SN_RA) + gt2r*(time5+time6)*(RA+SN_RA) + 
gt1s*(time3+time4)*SLE + gt2s*(time5+time6)*SLE + t1; 

mu_pi = b0 + br*(RA+SN_RA) + bs*SLE + ba*age + 
bt1*(time3+time4) + bt2*(time5+time6) + 
bt1r*(time3+time4)*(RA+SN_RA) + bt2r*(time5+time6)*(RA+SN_RA) + 
bt1s*(time3+time4)*SLE + bt2s*(time5+time6)*SLE + t2; 

pi = CDF('normal', mu_pi);

w = sqrt(sigma**2 + delta**2);
if censored then p = pi;
else p = (1-pi)*2/w * PDF('normal',(y-mu)/w) * CDF('normal',delta/sigma*(y-mu)/w);
loglike = log(p);

model y ~ general (loglike);
random t1 t2 ~ normal([0,0],[s11, s12, s22]) subject = individual;
run;


/* TOBIT WITH SKEW */
ods html file="tobit.html" path="/folders/myfolders";
proc nlmixed
data = multiplex
cov
itdetails
df=1000
alpha=0.05
gconv = 0
tech= QUANEW
update = BFGS;
parms 
g0 = 2.41 ga = 0 gr = 0.33 gs = 0.53 gt1 = 0.91 gt2 = 0.92 
gt1r = -0.97 gt2r = -1.25 gt1s = -1.44 gt2s = -1.41
sigma = 0.51 delta = -1.61
s11 = 0.7;

bounds sigma >= 0; bounds s11 > 0; 

T = log(0.83); *LOD;
censored = (y=-100); *Indicator variable;

mu = g0 + gr*(RA+SN_RA) + gs*SLE + ga*age + 
gt1*(time3+time4) + gt2*(time5+time6) + 
gt1r*(time3+time4)*(RA+SN_RA) + gt2r*(time5+time6)*(RA+SN_RA) + 
gt1s*(time3+time4)*SLE + gt2s*(time5+time6)*SLE + t1; 

* Estimate Owenst T-function;
h = (T - mu)/sqrt(sigma**2 + delta**2);
a = delta/sigma;
jmax = 30;
cut.point = 8;

aa = abs(a);
ah = abs(h);
if (aa = .I) then OT = sign(a) * 0.5*CDF('normal',-ah);
else if (aa = 0) then OT = 0;
else if (ah = .I) then OT = 0;
else if (aa <= 1) then do;
	th = ah;
	ta = aa;
	
	* Two ways to approximate integral based on cutpoint;
	low = (th <= cut.point);
	if low then do; *th < 8, ta <= 1;
		matr = 0;
		cumb = 0;
			do j = 0 to jmax;
				cumb = cumb + (th**(2*j)) / ((2**j) * GAMMA(j+1));
				
				*b1;
				if(-0.5 * th**2 < -600) then b1 = 0;
				else if(-0.5 * th**2 > 600) then b1 = .I * cumb; 
				else b1 = EXP(-0.5 * th**2) * cumb;
				
				matr = matr + (-1)**j * ta**(2*j+1) / (2*j+1) * (1-b1);
			end; *for;
		T.int = (ATAN(ta) - matr) / (2*CONSTANT("pi"));
		OT = SIGN(a) * T.int;
		end; *if;
	else do; *th > 8, ta <= 1;
		if(-0.5*(th**2)*ta/ATAN(ta) < -600) then factor = 0;
		else if(-0.5*(th**2)*ta/ATAN(ta) > 600) then factor = .I; 
		else factor = EXP(-0.5*(th**2)*ta/ATAN(ta));
		
		T.int = ATAN(ta) * factor * (1+0.00868*(th*ta)**4) / (2*CONSTANT("pi"));
	end;
end; *if;
else do; *(aa > 1);
	th = aa * ah;
	ta = 1 / aa;
	
	* Two ways to approximate integral based on cutpoint;
	low = (th <= cut.point);
	if low then do; *th < 8, ta < 1;
		matr = 0;
		cumb = 0;
			do j = 0 to jmax;
				cumb = cumb + (th**(2*j)) / ((2**j) * GAMMA(j+1));
				if(-0.5 * th**2 < -709) then b1 = 0;
				else if(-0.5 * th**2 > 709) then b1 = .I * cumb; 
				else b1 = EXP(-0.5 * th**2) * cumb;
				matr = matr + (-1)**j * ta**(2*j+1) / (2*j+1) * (1-b1);
			end; *for;
		T.int = (ATAN(ta) - matr) / (2*CONSTANT("pi"));
		end; *if;
	else do; *th >>>> 8, ta < 1;
		if(-0.5*(th**2)*ta/ATAN(ta) < -600) then factor = 0;
		else if(-0.5*(th**2)*ta/ATAN(ta) > 600) then factor = .I; 
		else factor = EXP(-0.5*(th**2)*ta/ATAN(ta));
		T.int = ATAN(ta) * factor * (1+0.00868*(th*ta)**4) / (2*CONSTANT("pi"));
	end;

	OT = sign(a) * (0.5*CDF('normal',ah) + CDF('normal',aa*ah)*(0.5-CDF('normal',ah)) - T.int);
end;

w = sqrt(sigma**2 + delta**2);
if censored then p =  (CDF('normal',(T-mu)/w) - 2*OT);
else p = 2/w * PDF('normal',(y-mu)/w) * CDF('normal',delta/sigma*(y-mu)/w);
loglike = log(p);

model y ~ general (loglike);
random t1 ~ normal(0,s11) subject = individual;
run;


/* SUBSTITUTE WITH SKEW */
proc nlmixed
data = multiplex_sort
cov
df = 1000
alpha = 0.05
gconv = 0
tech= QUANEW
update = BFGS;
parms 
g0 = 2.41 ga = 0 gr = 0.33 gs = 0.53 gt1 = 0.91 gt2 = 0.92 
gt1r = -0.97 gt2r = -1.25 gt1s = -1.44 gt2s = -1.41
sigma = 0.51 delta = -1.61
s11 = 0.5;

bounds sigma >= 0; bounds s11 > 0; 

T = log(0.29); *LOD;
censored = (y=-100); *Indicator variable;

mu = g0 + gr*(RA+SN_RA) + gs*SLE + ga*age + 
gt1*(time3+time4) + gt2*(time5+time6) + 
gt1r*(time3+time4)*(RA+SN_RA) + gt2r*(time5+time6)*(RA+SN_RA) + 
gt1s*(time3+time4)*SLE + gt2s*(time5+time6)*SLE + t1; 

w = sqrt(sigma**2 + delta**2);
if censored then p = 2/w * PDF('normal',(T-log(2)-mu)/w) * CDF('normal',delta/sigma*(T-log(2)-mu)/w);
else p = 2/w * PDF('normal',(y-mu)/w) * CDF('normal',delta/sigma*(y-mu)/w);
loglike = log(p);

model y ~ general (loglike);
random t1 ~ normal(0,s11) subject = individual;
run;


/* TPIC*/
proc nlmixed
data = multiplex_sort
cov
itdetails
df=1000
alpha=0.05
gconv = 0
tech= QUANEW
update = BFGS;
parms
g0 = 2.41 ga = 0 gr = 0.33 gs = 0.53 gt1 = 0.91 gt2 = 0.92 
gt1r = -0.97 gt2r = -1.25 gt1s = -1.44 gt2s = -1.41
sigma = 0.51 delta = -1.61
b0 = -0.67 br = -0.95 bs = -0.53 bt1 = -1.03 bt2 = -0.34  ba = 0.02
bt1r = 1.17 bt2r = -0.37 bt1s = 0.94 bt2s = -0.05
s11 = 0.5 s12 = 0 s22 = 0.5;

bounds sigma > 0; bounds s11 > 0; bounds s22 > 0;

T = log(0.29); *LOD;
censored = (y=-100); *Indicator variable;

mu = g0 + gr*(RA+SN_RA) + gs*SLE + ga*age + 
gt1*(time3+time4) + gt2*(time5+time6) + 
gt1r*(time3+time4)*(RA+SN_RA) + gt2r*(time5+time6)*(RA+SN_RA) + 
gt1s*(time3+time4)*SLE + gt2s*(time5+time6)*SLE + t1; 

mu_pi = b0 + br*(RA+SN_RA) + bs*SLE + ba*age + 
bt1*(time3+time4) + bt2*(time5+time6) + 
bt1r*(time3+time4)*(RA+SN_RA) + bt2r*(time5+time6)*(RA+SN_RA) + 
bt1s*(time3+time4)*SLE + bt2s*(time5+time6)*SLE + t2; 

pi = CDF('normal', mu_pi);

if censored then p = pi + (1-pi)*CDF('normal',(T-mu)/sigma);
else p = (1-pi) * 1/sigma * PDF('normal',(y-mu)/sigma);
loglike = log(p);

model y ~ general (loglike);
random t1 t2 ~ normal([0,0],[s11, s12, s22]) subject = individual;
run;


/* TP*/
proc nlmixed
data = multiplex
cov
df=1000
alpha=0.05
gconv = 0
tech= QUANEW
update = BFGS;
parms
g0 = 2.41 ga = 0 gr = 0.33 gs = 0.53 gt1 = 0.91 gt2 = 0.92 
gt1r = -0.97 gt2r = -1.25 gt1s = -1.44 gt2s = -1.41
sigma = 0.51 delta = -1.61
b0 = -0.67 br = -0.95 bs = -0.53 bt1 = -1.03 bt2 = -0.34  ba = 0.02
bt1r = 1.17 bt2r = -0.37 bt1s = 0.94 bt2s = -0.05
s11 = 0.5 s12 = 0 s22 = 0.5;

bounds sigma > 0; bounds s11 > 0; bounds s22 > 0;

T = log(0.29); *LOD;
censored = (y=-100); *Indicator variable;

mu = g0 + gr*(RA+SN_RA) + gs*SLE + ga*age + 
gt1*(time3+time4) + gt2*(time5+time6) + 
gt1r*(time3+time4)*(RA+SN_RA) + gt2r*(time5+time6)*(RA+SN_RA) + 
gt1s*(time3+time4)*SLE + gt2s*(time5+time6)*SLE + t1; 

mu_pi = b0 + br*(RA+SN_RA) + bs*SLE + ba*age + 
bt1*(time3+time4) + bt2*(time5+time6) + 
bt1r*(time3+time4)*(RA+SN_RA) + bt2r*(time5+time6)*(RA+SN_RA) + 
bt1s*(time3+time4)*SLE + bt2s*(time5+time6)*SLE + t2; 

if censored then p = pi;
else p = (1-pi) * 1/sigma * PDF('normal',(y-mu)/sigma);
loglike = log(p);

model y ~ general (loglike);
random t1 t2 ~ normal([0,0],[s11, s12, s22]) subject = individual;
run;
